package org.n52.connectors.core;

import java.io.IOException;

import org.geotools.data.DataStore;
import org.geotools.data.FeatureReader;
import org.geotools.data.FeatureSource;
import org.geotools.data.FeatureWriter;
import org.geotools.data.LockingManager;
import org.geotools.data.Query;
import org.geotools.data.Transaction;
import org.geotools.feature.FeatureType;
import org.geotools.feature.SchemaException;
import org.geotools.filter.Filter;
import org.geotools.metadata.iso.MetadataEntity;

abstract public class AbstractFeatureConnector extends AbstractConnector implements DataStore {

	protected DataStore dataStore = null;
	
	abstract public MetadataEntity getMetadata(String s); 

	abstract public boolean isSupported(ConnectorRequest request); 

	public void createSchema(FeatureType type) throws IOException {
		dataStore.createSchema(type);

	}

	public void updateSchema(String s, FeatureType type) throws IOException {
		dataStore.updateSchema(s, type);

	}

	public String[] getTypeNames() throws IOException {
		return dataStore.getTypeNames();
		
	}

	public FeatureType getSchema(String s) throws IOException {
		return dataStore.getSchema(s);
	}

	public FeatureSource getView(Query query) throws IOException,
			SchemaException {
		return dataStore.getView(query);
	}

	public FeatureSource getFeatureSource(String s) throws IOException {
		return dataStore.getFeatureSource(s);
	}

	public FeatureReader getFeatureReader(Query query, Transaction transaction)
			throws IOException {
		
		return dataStore.getFeatureReader(query, transaction);
	}

	public FeatureWriter getFeatureWriter(String s, Filter filter,
			Transaction transaction) throws IOException {
		return dataStore.getFeatureWriter(s, filter, transaction);
	}

	public FeatureWriter getFeatureWriter(String s, Transaction transaction)
			throws IOException {
		return dataStore.getFeatureWriter(s, transaction);
	}

	public FeatureWriter getFeatureWriterAppend(String s, Transaction transaction)
			throws IOException {
		return dataStore.getFeatureWriterAppend(s, transaction);
	}

	public LockingManager getLockingManager() {
		return dataStore.getLockingManager();
	}
}