/*
 * $Id: ConnectorCreation.java,v 1.7 2005/10/13 12:37:26 windowsnt\martin Exp $
 * 
 * Copyright (C) ITC 2005 - International Institute for Geo-Information Science and Earth Observation
 *    
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package org.n52.connectors.core;

import org.n52.connectors.core.exceptions.IllegalConnectorRequestException;
import org.n52.connectors.core.exceptions.N52Exception;

/**
 * Creation of connectors by a factory class. The classes that implement this
 * interface are considered client side classes and provide the users of
 * connectors with a interface to the repository that stores the functionality
 * supported by the Connector framework. Implementers of the interface must be
 * able to fecth connectors from a registry.
 * 
 * @author martin Schouwenburg
 */
public interface ConnectorCreation {

	/**
	 * return one object which supports an interface with requested
	 * functionality. If more than one is found the first one will be returned.
	 * To be usable a object must be cast to the functionality interface it
	 * supports.
	 * 
	 * @param request
	 *            contains all information that is needed to find a suitable
	 *            connector
	 * @return a object that supports the request
	 * @throws IllegalConnectorRequestException
	 *             request could not be fullfilled
	 * @throws N52Exception
	 *             low level errors that are passed to higher level error
	 *             handling
	 */
	Object getServiceAdapter(ServiceRequest request) throws N52Exception;

	/**
	 * returns one or more objects which supports an interface with requested
	 * functionality
	 * 
	 * @param request
	 *            contains all information that is needed to find a suitable
	 *            connector
	 * @return a set of objects that all supports the request
	 */
	Object[] getServiceAdapters(ServiceRequest request);

	/**
	 * adds a certain registry to the set of registries that will be searched to
	 * find certain functionality
	 * 
	 * @param registry
	 *            a registry made locally to query for functionality
	 */
	void addRegistry(ConnectorTypeRegistry registry);
}
