/*
 * $Id: Dependency.java,v 1.3 2005/10/13 12:37:27 windowsnt\martin Exp $
 * 
 * Copyright (C) ITC 2005 - International Institute for Geo-Information Science and Earth Observation
 *    
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package org.n52.connectors.core;

import java.io.File;
import java.util.ArrayList;

/**
 * Describes the name and location of a resource a connector needs to be able to
 * properly create an object that supports the requested fucntionality. These
 * resources can be retreived by the connector and installed on the client
 * system. Note that resources themselves can be dependend on other resources.
 * 
 * @author martin schouwenburg
 * 
 */
public class Dependency {

	File dependencyFile;

	private ArrayList<Dependency> dependencies = null;

	public Dependency(String name) {
		dependencyFile = new File(name);
	}

	public File getDependencyFile() {
		return dependencyFile;
	}

	public ArrayList<Dependency> getDependencies() {
		return dependencies;
	}

	public void setDependencies(ArrayList<Dependency> dependencies) {
		this.dependencies = dependencies;
	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof Dependency) {
			Dependency dep = (Dependency) o;
			if (dep.getDependencyFile().equals(dependencyFile)) {
				ArrayList<Dependency> secondaryDependencies = dep
						.getDependencies();
				for (Dependency d : dependencies) {
					if (!secondaryDependencies.contains(d)) {
						return false;
					}
				}
				return true;
			}

		}

		return false;
	}

}
