/*
 * $Id: ServiceRequest.java,v 1.2 2005/10/18 06:46:43 windowsnt\martin Exp $
 * 
 * Copyright (C) ITC 2005 - International Institute for Geo-Information Science and Earth Observation
 *    
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package org.n52.connectors.core;

import java.util.HashMap;

import org.n52.connectors.core.exceptions.ParameterDoesNotExistException;

/**
 * In the connector request a user formulates which functionality he wants from
 * the service provider. This can be as simple as requesting a direct name of
 * defined functionality to starting a discovery process to find certain
 * functionality in the offered connectors (e.g. via keywords)
 * 
 * @author martin schouwenburg
 * 
 */
public class ServiceRequest {
	/**
	 * 
	 */
	private static final long serialVersionUID = 2284089924224561238L;
	private HashMap<String, Object> parameters = new HashMap<String, Object>();

	public ServiceRequest() {
		put("connector", "default");
	}

	/**
	 * Gives a certain parameter a certain value. Predefined parametersId's are
	 * service : The name of the service you want to access connector : The
	 * specific implementation of a connector. optional; it normally chooses a
	 * predfined default resource : defines a data source; can be local or on
	 * the web
	 * 
	 * @param parameterId
	 *            the name of the parameter
	 * @param value
	 *            the value of the parameter
	 */
	public Object put(String parameterId, Object value) {
		return parameters.put(parameterId, value);
	}

	/**
	 * Returns the value that is indicated by parameterId
	 * 
	 * @param parameterId
	 * @return specific value as string
	 * @throws ParameterDoesNotExistException
	 *             if a parameter is asked that does not exist
	 */
	public Object get(String parameterId) throws ParameterDoesNotExistException {
		if (!parameters.containsKey(parameterId))
			throw new ParameterDoesNotExistException("Parameter " + parameterId
					+ " can not be found in the request");

		return parameters.get(parameterId);
	}

}
