package org.n52.connectors.impl;

import java.awt.image.renderable.RenderableImage;
import java.util.List;
import java.util.Locale;

import org.geotools.metadata.iso.MetadataEntity;
import org.n52.connectors.core.AbstractConnector;
import org.n52.connectors.core.ConnectorRequest;
import org.n52.connectors.core.exceptions.IllegalConnectorRequest;
import org.n52.connectors.core.exceptions.ParameterDoesNotExist;
import org.opengis.coverage.CannotEvaluateException;
import org.opengis.coverage.MetadataNameNotFoundException;
import org.opengis.coverage.SampleDimension;
import org.opengis.coverage.grid.GridCoverage;
import org.opengis.coverage.grid.GridGeometry;
import org.opengis.coverage.grid.GridNotEditableException;
import org.opengis.coverage.grid.GridPacking;
import org.opengis.coverage.grid.GridRange;
import org.opengis.coverage.grid.InvalidRangeException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.spatialschema.geometry.DirectPosition;
import org.opengis.spatialschema.geometry.Envelope;
import org.opengis.util.InternationalString;
import org.w3c.dom.Document;

public class GdalConnector extends AbstractConnector implements GridCoverage {

	public GdalConnector() {
		connectorName = "itc.GdalConnector";
	}

	@Override
	public boolean isSupported(ConnectorRequest request) {
		boolean supported = false;
		long previousGdalDataSet = gdalDataSet; // openGdal replaces the
		// gdalDataSet
		try {
			boolean canDo = new String("gs.open.gridcoverage")
					.equalsIgnoreCase((String) request.get("service"));
			if (canDo) {
				String fileName = (String) request.get("resource");
				String access = (String) request.get("accessmode");
				if (access.compareToIgnoreCase("read") == 0)
					openGdal(fileName, false);
				else if (access.compareToIgnoreCase("readwrite") == 0)
					openGdal(fileName, true);
				else if (access.compareToIgnoreCase("create") == 0) {
					String driver = (String) request.get("driver");
					String datatype = (String) request.get("datatype");
					String pixels = (String) request.get("xsize");
					String lines = (String) request.get("ysize");
					String bands = (String) request.get("bands");
					String srs_wkt = (String) request.get("srs_wkt");
					double[] adfGeoTransform = new double[6];
					for (int i = 0; i < 6; ++i) {
						String strGeoTransform = (String) request
								.get("geotransform" + i);
						adfGeoTransform[i] = Double
								.parseDouble(strGeoTransform);
					}
					int nPixels = Integer.parseInt(pixels);
					int nLines = Integer.parseInt(lines);
					int nBands = Integer.parseInt(bands);
					createGdal(fileName, driver, datatype, nPixels, nLines,
							nBands, srs_wkt, adfGeoTransform);
				}
				// if we reached here, there is no throw, so we conclude that
				// the
				// request is supported
				supported = true;
			}
		} catch (IllegalConnectorRequest e) {
		} catch (ParameterDoesNotExist e) {
		}
		closeGdal();
		gdalDataSet = previousGdalDataSet; // restore the original ptr

		return supported;
	}

	public MetadataEntity getMetadata(String s) {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean isDataEditable() {
		// TODO Auto-generated method stub
		return false;
	}

	public GridPacking getGridPacking() {
		// TODO Auto-generated method stub
		return null;
	}

	public GridGeometry getGridGeometry() {
		// TODO Auto-generated method stub
		return null;
	}

	public int[] getOptimalDataBlockSizes() {
		// TODO Auto-generated method stub
		return null;
	}

	public int getNumOverviews() {
		// TODO Auto-generated method stub
		return 0;
	}

	public GridGeometry getOverviewGridGeometry(int arg0)
			throws IndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public GridCoverage getOverview(int arg0) throws IndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean[] getDataBlock(GridRange arg0, boolean[] arg1)
			throws InvalidRangeException, ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public byte[] getDataBlock(GridRange arg0, byte[] arg1)
			throws InvalidRangeException, ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public short[] getDataBlock(GridRange arg0, short[] arg1)
			throws InvalidRangeException, ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public int[] getDataBlock(GridRange arg0, int[] arg1)
			throws InvalidRangeException, ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public float[] getDataBlock(GridRange arg0, float[] arg1)
			throws InvalidRangeException, ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public double[] getDataBlock(GridRange arg0, double[] arg1)
			throws InvalidRangeException, ArrayIndexOutOfBoundsException {
		int xStart = arg0.getLower(0);
		int yStart = arg0.getLower(1);
		int xSize = arg0.getLength(0);
		int ySize = arg0.getLength(1);
		int nBands = 1;
		return getDataBlock(xStart, yStart, xSize, ySize, nBands, arg1);
	}

	public native double[] getDataBlock(int xStart, int yStart, int xSize,
			int ySize, int dimensions, double[] destination)
			throws InvalidRangeException;

	public byte[] getPackedDataBlock(GridRange arg0)
			throws InvalidRangeException {
		// TODO Auto-generated method stub
		return null;
	}

	public void setDataBlock(GridRange arg0, boolean[] arg1)
			throws InvalidRangeException, GridNotEditableException,
			ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub

	}

	public void setDataBlock(GridRange arg0, byte[] arg1)
			throws InvalidRangeException, GridNotEditableException,
			ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub

	}

	public void setDataBlock(GridRange arg0, short[] arg1)
			throws InvalidRangeException, GridNotEditableException,
			ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub

	}

	public void setDataBlock(GridRange arg0, int[] arg1)
			throws InvalidRangeException, GridNotEditableException,
			ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub

	}

	public void setDataBlock(GridRange arg0, float[] arg1)
			throws InvalidRangeException, GridNotEditableException,
			ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub

	}

	public void setDataBlock(GridRange arg0, double[] arg1)
			throws InvalidRangeException, GridNotEditableException,
			ArrayIndexOutOfBoundsException {
		int xStart = arg0.getLower(0);
		int yStart = arg0.getLower(1);
		int xSize = arg0.getLength(0);
		int ySize = arg0.getLength(1);
		int nBands = 1;
		setDataBlock(xStart, yStart, xSize, ySize, nBands, arg1);
	}

	public native void setDataBlock(int xStart, int yStart, int xSize,
			int ySize, int dimensions, double[] values)
			throws InvalidRangeException, GridNotEditableException;

	public void setPackedDataBlock(GridRange arg0, byte[] arg1)
			throws InvalidRangeException, GridNotEditableException,
			ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub

	}

	public CoordinateReferenceSystem getCoordinateReferenceSystem() {
		// TODO Auto-generated method stub
		return null;
	}

	public Envelope getEnvelope() {
		// TODO Auto-generated method stub
		return null;
	}

	public String[] getDimensionNames(Locale arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	public int getNumSampleDimensions() {
		// TODO Auto-generated method stub
		return 0;
	}

	public SampleDimension getSampleDimension(int arg0)
			throws IndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public List getSources() {
		// TODO Auto-generated method stub
		return null;
	}

	public String[] getMetadataNames() {
		// TODO Auto-generated method stub
		return null;
	}

	public String getMetadataValue(String arg0)
			throws MetadataNameNotFoundException {
		// TODO Auto-generated method stub
		return null;
	}

	public Object evaluate(DirectPosition arg0) throws CannotEvaluateException {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean[] evaluate(DirectPosition arg0, boolean[] arg1)
			throws CannotEvaluateException, ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public byte[] evaluate(DirectPosition arg0, byte[] arg1)
			throws CannotEvaluateException, ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public int[] evaluate(DirectPosition arg0, int[] arg1)
			throws CannotEvaluateException, ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public float[] evaluate(DirectPosition arg0, float[] arg1)
			throws CannotEvaluateException, ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public double[] evaluate(DirectPosition arg0, double[] arg1)
			throws CannotEvaluateException, ArrayIndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public RenderableImage getRenderableImage(int arg0, int arg1)
			throws UnsupportedOperationException, IndexOutOfBoundsException {
		// TODO Auto-generated method stub
		return null;
	}

	public InternationalString[] getDimensionNames() {
		// TODO Auto-generated method stub
		return null;
	}

	public void factorySideInitialization(Document xmlConfiguration,
			ConnectorRequest request) throws IllegalConnectorRequest,
			ParameterDoesNotExist {
		String fileName = (String) request.get("resource");
		String access = (String) request.get("accessmode");
		if (access.compareToIgnoreCase("read") == 0)
			openGdal(fileName, false);
		else if (access.compareToIgnoreCase("readwrite") == 0)
			openGdal(fileName, true);
		else if (access.compareToIgnoreCase("create") == 0) {
			String driver = (String) request.get("driver");
			String datatype = (String) request.get("datatype");
			String pixels = (String) request.get("xsize");
			String lines = (String) request.get("ysize");
			String bands = (String) request.get("bands");
			String srs_wkt = (String) request.get("srs_wkt");
			double[] adfGeoTransform = new double[6];
			for (int i = 0; i < 6; ++i) {
				String strGeoTransform = (String) request.get("geotransform"
						+ i);
				adfGeoTransform[i] = Double.parseDouble(strGeoTransform);
			}
			int nPixels = Integer.parseInt(pixels);
			int nLines = Integer.parseInt(lines);
			int nBands = Integer.parseInt(bands);
			createGdal(fileName, driver, datatype, nPixels, nLines, nBands,
					srs_wkt, adfGeoTransform);
		}
	}

	public String registrySideInitialization(String returnXml,
			ConnectorRequest request) {
		// TODO Auto-generated method stub
		return null;
	}

	private native void openGdal(String fileName, boolean readWrite)
			throws IllegalConnectorRequest;

	private native void createGdal(String fileName, String driver,
			String datatype, int pixels, int lines, int bands, String srs_wkt,
			double[] adfGeoTransform) throws IllegalConnectorRequest;

	private native void closeGdal(); // close after every call to openGdal or

	// createGdal

	// call close() for every call of initialize() on this driver
	public void close() {
		closeGdal();
	}

	private long gdalDataSet = 0; // used in the "native" part to store the
	// gdalDataSet

	static {
		System.loadLibrary("GdalConnector");
	}
}
