package org.n52.connectors.impl;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;

import org.geotools.metadata.iso.MetadataEntity;
import org.n52.connectors.core.Dependency;
import org.n52.connectors.core.ServiceRequest;
import org.n52.connectors.core.exceptions.IllegalConnectorRequestException;
import org.n52.connectors.core.exceptions.ParameterDoesNotExistException;
import org.w3c.dom.Document;
import org.geotools.data.wfs.WFSDataStoreFactory;

public class WFSConnector extends FeatureConnector {

	public WFSConnector() {
		connectorName = "itc.wfsconnector";
		addDependency( new Dependency("gt2-wfs.jar"));
		addDependency(new Dependency("vecmath-1.3.jar"));
		addDependency(new Dependency("unit-0.01.jar"));		
		addDependency(new Dependency("geoapi-2.0.jar"));
		addDependency( new Dependency("gt2-main.jar"));		
	}

	@Override
	public MetadataEntity getMetadata(String s) {
		return null;
	}

	@Override
	public boolean isSupported(ServiceRequest request) {
		boolean canDo = false;
		try {
			 canDo = new String("gs.open.features").equalsIgnoreCase((String)request.get("service"));
			 URL source = new URL((String) request.get("resource"));
			 // naive check for being WFS. must be improved in the future
			 String query = source.getQuery();
			 if ( query.toLowerCase().lastIndexOf("getcapabilities") == -1)
				 canDo = false;
		} catch (ParameterDoesNotExistException e) {
			canDo = false;
		} catch (MalformedURLException e) {
			canDo = false;
		}
		return canDo;
	}

	@Override
	public void factorySideInitialization(Document xmlInit, ServiceRequest request) throws IllegalConnectorRequestException {
		HashMap<String, Object> parameters = new HashMap<String, Object>();

		try {
			Integer timeout = new Integer(10000);
			Boolean protocol = Boolean.FALSE;
			String source = "";
			
			try {source = (String) request.get("resource");
			} catch (ParameterDoesNotExistException e1) {}
			
			try { timeout = (Integer) request.get("timeout");
			} catch (ParameterDoesNotExistException e) {}
			
			try { protocol = (Boolean) request.get("protocol");
			} catch (ParameterDoesNotExistException e) {	}
			
			parameters.put(WFSDataStoreFactory.URL.key, source);
			parameters.put(WFSDataStoreFactory.TIMEOUT.key,
					new Integer(timeout));
			parameters.put(WFSDataStoreFactory.PROTOCOL.key, new Boolean(
					protocol));

			dataStore = (new WFSDataStoreFactory())
					.createNewDataStore(parameters);

		} catch (IOException e) {
			// throw new WrappedException(e);
		} 

	}
}
