package org.n52.connectors.testcases;

import junit.framework.TestCase;

import org.geotools.data.DataStore;
import org.n52.connectors.core.ServiceFactory;
import org.n52.connectors.core.ServiceRequest;
import org.n52.connectors.core.ConnectorTypeRegistry;
import org.n52.connectors.core.LocalConnectorTypeRegistry;
import org.n52.connectors.core.exceptions.N52Exception;
import org.n52.connectors.impl.TestServiceObject;

public class ConnectorFactoryTester extends TestCase {

	private ConnectorTypeRegistry registry;
	private ServiceFactory factory;

	/*
	 * Test method for 'org.n52.connectors.core.AbstractConnectorTypeRegistry.getConnectorClass(ConnectorRequest)'
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
		registry = new LocalConnectorTypeRegistry("D:/EclipseProjects/CoreConnectors/ConnectorImpl");
		factory = new ServiceFactory();
		factory.addRegistry(registry);		
	}
	
	public void testGetConnectorClassCorrect() {
		try {

			ServiceRequest request = new ServiceRequest();
			request.put("service", "test.test");
			TestServiceObject conn = (TestServiceObject)factory.getServiceAdapter(request);
			assertEquals(conn.test(), "42");
		} catch (N52Exception e) {
			assertTrue(true);
		}
	}
	
	/**
	 * request feature connector
	 */
	public void testGetConnectorClassWFS() {
		try {
			ServiceRequest request = new ServiceRequest();
			request.put("service", "gs.open.features");
			request.put("resource", "http://www.refractions.net:8080/geoserver/wfs?REQUEST=GetCapabilities");
			DataStore conn = (DataStore)factory.getServiceAdapter(request);
			assertEquals(conn.getClass().getName(), "WFSConnector");
		} catch (N52Exception e) {
			assertTrue(false);
		}
	}
	
	public void testGetConnectorClassGdal() {
//		try {
//			ConnectorRequest request = new ConnectorRequest();
//			request.put("service", "gs.open.gridcoverage");
//			request.put("resource", "c:/temp/laea.tif");
//			request.put("accessmode", "read");			
//			GdalConnector conn = (GdalConnector)factory.getServiceObject(request);
//			assertEquals(conn.getName(), "itc.GdalConnector");
//		} catch (N52Exception e) {
//			assertTrue(false);
//		}
	}
	
	public void testGetConnectorClassGdalAccess() {
//		try {
//			ConnectorRequest request = new ConnectorRequest();
//			request.put("service", "gs.open.gridcoverage");
//			request.put("resource", "c:/temp/laea.tif");
//			request.put("accessmode", "read");			
//			GdalConnector conn = (GdalConnector)factory.getServiceObject(request);
//			GridCoverage gCoverage = (GridCoverage)conn;
//			double[] grid = new double[300 * 300];
//			gCoverage.getDataBlock(new GeneralGridRange( new Rectangle(10,10, 290, 290)), grid);
//			double v = grid[10];
//  		System.out.println(v);
//			
//		} catch (N52Exception e) {
//			assertTrue(false);
//		}
	}	
	
	/**
	 * request unknown service; must give exception
	 */
	public void testGetConnectorClassInCorrect() {
//		try {
//			ConnectorRequest request = new ConnectorRequest();
//			request.put("service", "zzz");
////			GdalConnector conn = (GdalConnector)factory.getServiceObject(request);
////			assertEquals(conn.getClass().getName(), "GdalConnector");
//		} catch (N52Exception e) {
//			assertTrue(true);
//		}
	}
	
	/**
	 * pass unknown command; must give exception 
	 */
	public void testGetRequestInCorrect() {
//		try {
//			ConnectorTypeRegistry registry = new LocalConnectorTypeRegistry("");
//			ConnectorFactory factory = new ConnectorFactory();
//			factory.addRegistry(registry);
//			ConnectorRequest request = new ConnectorRequest();
//			request.put("servicexxx", "test.test");
////			GdalConnector conn = (GdalConnector)factory.getServiceObject(request);
////			assertEquals(conn.getClass().getName(), "GdalConnector");
//		} catch (N52Exception e) {
//			assertTrue(true);
//		}
	}	

	/*
	 * Test method for 'org.n52.connectors.core.AbstractConnectorTypeRegistry.setDefaultConnector(String, String)'
	 */
	public void testSetDefaultConnector() {

	}

}
